/*
 * Copyright (C) 2006-2008 Intel Corporation. All Rights Reserved.
 *
 * The source code contained or described herein and all
 * documents related to the source code ("Material") are owned by
 * Intel Corporation or its suppliers or licensors. Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors. The Material is protected by worldwide copyright
 * laws and treaty provisions.  No part of the Material may be
 * used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed,  or disclosed in any way
 * except as expressly provided in the license provided with the
 * Materials.  No license under any patent, copyright, trade
 * secret or other intellectual property right is granted to or
 * conferred upon you by disclosure or delivery of the Materials,
 * either expressly, by implication, inducement, estoppel or
 * otherwise, except as expressly provided in the license
 * provided with the Materials.
 *
 * [DESCRIPTION]
 * Demonstrates functions that are not able to be vectorized,
 * and generates only the object file.
 *
 * Note: This sample is not meant to generate an executable.  
 *
 * [COMPILE]  
 * Compile on IA-32 and Intel(R) 64 architectures only. 
 * Use the following compiler options to compile: 
 *
 * Windows*: /c /Qstd=c99 /QxSSSE3 /Qvec-report:3
 *  
 * Linux* and Mac OS* X: -c -std=c99 -xSSSE3 -vec-report3
 *  
*/

#define M 1024
void novec_cse(double a[][M], double b[][M],double c[][M])
{
  // Outer loops do not vectorize.
  for (int i=0; i<M; i++)
    for (int k=0; k<M; k++)
      // Cannot vectorize due to flow- and anti-depedences.
      for (int j=0; j<M; j++)
        c[i][j] = c[i][j] + a[i][k] * b[k+1][j+1];
}

void novec_flow(double *y)
{
  // Cannot vectorize due to dependence.
  for (int i=1; i<10; i++)
    y[i] = y[i-1] + 1;
}

#include <math.h>
void novec_mixed_types(double *x, double *y)
{
  int within_border = 0; // change to "double" to vectorize
  double dist;
  // Cannot vectorize due to mixed data types in loop.
  for (int i=0; i<100; i++) {
    dist = sqrtf(x[i]*x[i] + y[i]*y[i]);
    if (dist<5)
      within_border++;
  }
}

static int func(int n)
{ return (n*n-1); }
void unsupported_loop_structure(double *y, int n)
{
  // Cannot vectorize due to non-determinable function call result.
  for (int i=0; i<func(n); i++)
    *y = *y * 2.0;
}
