/*
 * Copyright (C) 2006-2008 Intel Corporation. All Rights Reserved.
 *
 * The source code contained or described herein and all
 * documents related to the source code ("Material") are owned by
 * Intel Corporation or its suppliers or licensors. Title to the
 * Material remains with Intel Corporation or its suppliers and
 * licensors. The Material is protected by worldwide copyright
 * laws and treaty provisions.  No part of the Material may be
 * used, copied, reproduced, modified, published, uploaded,
 * posted, transmitted, distributed,  or disclosed in any way
 * except as expressly provided in the license provided with the
 * Materials.  No license under any patent, copyright, trade
 * secret or other intellectual property right is granted to or
 * conferred upon you by disclosure or delivery of the Materials,
 * either expressly, by implication, inducement, estoppel or
 * otherwise, except as expressly provided in the license
 * provided with the Materials.
 *
 * [DESCRIPTION]
 * Demonstrates functions that are able to be vectorized,
 * and generates only the object file.
 *
 * Note: This sample is not meant to generate an executable. 
 *
 * [COMPILE]  
 * Compile on IA-32 and Intel(R) 64 architectures only. 
 * Use the following compiler options to compile: 
 *  
 * Windows*: /c /Qstd=c99 /QxSSSE3 /Qrestrict /Qvec-report:3
 *  
 * Linux* and Mac OS* X: -c -std=c99 -xSSSE3 -restrict -vec-report3
 *
*/

void vec(double *y)
{
  // Vectorize!
  for (int i=1; i<10; i++)
    y[i] = y[0] + i;
}

void vec_copy_multi_version(float *dest, float *src, int len)
{
  // Vectorizable copy due to unproven distinction.
  for (int i=0; i<len; i++)
    dest[i] = src[i];
}

// Using restrict to prove vectorizable distinction.
void vec_copy(float *restrict dest, float *restrict src, int len)
{
  for (int i=0; i<len; i++)
    dest[i] = src[i];
}

void aligned(float *a, float *b, int len)
{
  for (int i=0; i<len; i++)
    a[i] = b[i];
}

void unaligned(int lb, float *a, float x, float *y, int len)
{
  // Unaligned due to unknown variable value at compile time.
  // (But still vectorized.)
  for (int i=lb; i<len; i++)
    a[i] = a[i] * x + y[i];
}

#include <assert.h>
void assert_aligned(int lb, float *a, float x, float *y, int len)
{
  assert(lb%4 == 0);
  #pragma vector aligned
  // Aligned thanks to assertion of variable as multiple of 4.
  for (int i=lb; i<len; i++)
    a[i] = a[i] * x + y[i];
}
